@@ -140,6 +140,8 @@ module ActiveLdap
       def read_external_file
         uri_string = @scanner.scan(URI::ABS_URI)
         raise uri_is_missing if uri_string.nil?
+        uri_string.chomp!
+
         uri = nil
         begin
           uri = URI.parse(uri_string)
@@ -441,6 +443,10 @@ module ActiveLdap
         invalid_ldif(_("URI is invalid: %s: %s") % [uri_string, message])
       end
 
+      def uri_is_missing
+        invalid_ldif(_("URI is missing"))
+      end
+
       def modify_spec_separator_is_missing
         invalid_ldif(_("'-' is missing"))
       end
